/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.jenkins.results.parser;

/**
 * @author Michael Hashimoto
 */
public class PortalBatchBuildRunner extends BatchBuildRunner {

	@Override
	public void setup() {
		primaryLocalRepository.setup();

		writeRepositoryProperties();
	}

	protected PortalBatchBuildRunner(Job job, String batchName) {
		super(job, batchName);

		if (!(job instanceof PortalTestClassJob)) {
			throw new RuntimeException("Invalid job type");
		}

		PortalTestClassJob portalTestClassJob = (PortalTestClassJob)job;

		PortalGitWorkingDirectory portalGitWorkingDirectory =
			portalTestClassJob.getPortalGitWorkingDirectory();

		primaryLocalRepository = RepositoryFactory.getLocalRepository(
			portalGitWorkingDirectory.getRepositoryName(),
			portalGitWorkingDirectory.getUpstreamBranchName());

		if (!(primaryLocalRepository instanceof PortalLocalRepository)) {
			throw new RuntimeException("Invalid workspace");
		}

		portalLocalRepository = (PortalLocalRepository)primaryLocalRepository;

		_setPortalJobBuildProperties();
	}

	protected void writeRepositoryProperties() {
		portalLocalRepository.writeRepositoryPropertiesFiles();
	}

	protected final PortalLocalRepository portalLocalRepository;

	private void _setPortalJobBuildProperties() {
		portalLocalRepository.setBuildProperties(getPortalJobBuildProperties());
	}

}